@@ -22,6 +22,7 @@ nick=NICKNAME
 pass=PASSWORD
 server=irc.freenode.net
 port=6667
+server_password=SERVERPASS
 channel_config=/path/to/yaml/config
 
 [gerrit]
@@ -49,6 +50,8 @@ import daemon
 import irc.bot
 import logging.config
 import os
+import re
+import ssl
 import sys
 import threading
 import time
@@ -57,18 +60,30 @@ import yaml
 try:
     import daemon.pidlockfile
     pid_file_module = daemon.pidlockfile
-except:
+except Exception:
     # as of python-daemon 1.6 it doesn't bundle pidlockfile anymore
     # instead it depends on lockfile-0.9.1
     import daemon.pidfile
     pid_file_module = daemon.pidfile
 
 
+# https://bitbucket.org/jaraco/irc/issue/34/
+# irc-client-should-not-crash-on-failed
+# ^ This is why pep8 is a bad idea.
+irc.client.ServerConnection.buffer_class.errors = 'replace'
+
+
 class GerritBot(irc.bot.SingleServerIRCBot):
-    def __init__(self, channels, nickname, password, server, port=6667):
-        irc.bot.SingleServerIRCBot.__init__(self,
-                                           [(server, port)],
-                                           nickname, nickname)
+    def __init__(self, channels, nickname, password, server, port=6667,
+                 server_password=None):
+        if port == 6697:
+            factory = irc.connection.Factory(wrapper=ssl.wrap_socket)
+            super(GerritBot, self).__init__([(server, port, server_password)],
+                                            nickname, nickname,
+                                            connect_factory=factory)
+        else:
+            super(GerritBot, self).__init__([(server, port, server_password)],
+                                            nickname, nickname)
         self.channel_list = channels
         self.nickname = nickname
         self.password = password
@@ -91,17 +106,22 @@ class GerritBot(irc.bot.SingleServerIRCBot):
         for channel in self.channel_list:
             c.join(channel)
             self.log.info('Joined channel %s' % channel)
+            time.sleep(0.5)
 
     def send(self, channel, msg):
         self.log.info('Sending "%s" to %s' % (msg, channel))
-        self.connection.privmsg(channel, msg)
-        time.sleep(0.5)
+        try:
+            self.connection.privmsg(channel, msg)
+            time.sleep(0.5)
+        except Exception:
+            self.log.exception('Exception sending message:')
+            self.reconnect()
 
 
 class Gerrit(threading.Thread):
     def __init__(self, ircbot, channel_config, server,
                  username, port=29418, keyfile=None):
-        threading.Thread.__init__(self)
+        super(Gerrit, self).__init__()
         self.ircbot = ircbot
         self.channel_config = channel_config
         self.log = logging.getLogger('gerritbot')
@@ -115,19 +135,19 @@ class Gerrit(threading.Thread):
         # Import here because it needs to happen after daemonization
         import gerritlib.gerrit
         try:
-            self.gerrit = gerritlib.gerrit.Gerrit(self.server, self.username,
-                                                    self.port, self.keyfile)
+            self.gerrit = gerritlib.gerrit.Gerrit(
+                self.server, self.username, self.port, self.keyfile)
             self.gerrit.startWatching()
             self.log.info('Start watching Gerrit event stream.')
             self.connected = True
-        except:
+        except Exception:
             self.log.exception('Exception while connecting to gerrit')
             self.connected = False
             # Delay before attempting again.
             time.sleep(1)
 
     def patchset_created(self, channel, data):
-        msg = '%s proposed a change to %s: %s  %s' % (
+        msg = '%s proposed %s: %s  %s' % (
             data['patchSet']['uploader']['name'],
             data['change']['project'],
             data['change']['subject'],
@@ -135,6 +155,20 @@ class Gerrit(threading.Thread):
         self.log.info('Compiled Message %s: %s' % (channel, msg))
         self.ircbot.send(channel, msg)
 
+    def ref_updated(self, channel, data):
+        refName = data['refUpdate']['refName']
+        m = re.match(r'(refs/tags)/(.*)', refName)
+
+        if m:
+            tag = m.group(2)
+            msg = '%s tagged project %s with %s' % (
+                data['submitter']['username'],
+                data['refUpdate']['project'],
+                tag
+            )
+            self.log.info('Compiled Message %s: %s' % (channel, msg))
+            self.ircbot.send(channel, msg)
+
     def comment_added(self, channel, data):
         msg = 'A comment has been added to a proposed change to %s: %s  %s' % (
             data['change']['project'],
@@ -144,9 +178,9 @@ class Gerrit(threading.Thread):
         self.ircbot.send(channel, msg)
 
         for approval in data.get('approvals', []):
-            if (approval['type'] == 'VRIF' and approval['value'] == '-2' and
-                                channel in self.channel_config.events.get(
-                                                    'x-vrif-minus-2', set())):
+            if (approval['type'] == 'VRIF' and approval['value'] == '-2'
+                and channel in self.channel_config.events.get(
+                    'x-vrif-minus-2', set())):
                 msg = 'Verification of a change to %s failed: %s  %s' % (
                     data['change']['project'],
                     data['change']['subject'],
@@ -154,9 +188,9 @@ class Gerrit(threading.Thread):
                 self.log.info('Compiled Message %s: %s' % (channel, msg))
                 self.ircbot.send(channel, msg)
 
-            if (approval['type'] == 'VRIF' and approval['value'] == '2' and
-                                channel in self.channel_config.events.get(
-                                                    'x-vrif-plus-2', set())):
+            if (approval['type'] == 'VRIF' and approval['value'] == '2'
+                and channel in self.channel_config.events.get(
+                    'x-vrif-plus-2', set())):
                 msg = 'Verification of a change to %s succeeded: %s  %s' % (
                     data['change']['project'],
                     data['change']['subject'],
@@ -164,9 +198,9 @@ class Gerrit(threading.Thread):
                 self.log.info('Compiled Message %s: %s' % (channel, msg))
                 self.ircbot.send(channel, msg)
 
-            if (approval['type'] == 'CRVW' and approval['value'] == '-2' and
-                                channel in self.channel_config.events.get(
-                                                    'x-crvw-minus-2', set())):
+            if (approval['type'] == 'CRVW' and approval['value'] == '-2'
+                and channel in self.channel_config.events.get(
+                    'x-crvw-minus-2', set())):
                 msg = 'A change to %s has been rejected: %s  %s' % (
                     data['change']['project'],
                     data['change']['subject'],
@@ -174,9 +208,9 @@ class Gerrit(threading.Thread):
                 self.log.info('Compiled Message %s: %s' % (channel, msg))
                 self.ircbot.send(channel, msg)
 
-            if (approval['type'] == 'CRVW' and approval['value'] == '2' and
-                                channel in self.channel_config.events.get(
-                                                    'x-crvw-plus-2', set())):
+            if (approval['type'] == 'CRVW' and approval['value'] == '2'
+                and channel in self.channel_config.events.get(
+                    'x-crvw-plus-2', set())):
                 msg = 'A change to %s has been approved: %s  %s' % (
                     data['change']['project'],
                     data['change']['subject'],
@@ -185,7 +219,7 @@ class Gerrit(threading.Thread):
                 self.ircbot.send(channel, msg)
 
     def change_merged(self, channel, data):
-        msg = 'A change was merged to %s: %s  %s' % (
+        msg = 'Merged %s: %s  %s' % (
             data['change']['project'],
             data['change']['subject'],
             data['change']['url'])
@@ -194,18 +228,21 @@ class Gerrit(threading.Thread):
 
     def _read(self, data):
         try:
-            channel_set = (self.channel_config.projects.get(
-                                data['change']['project'], set()) &
-                            self.channel_config.events.get(
-                                data['type'], set()) &
-                            self.channel_config.branches.get(
-                                data['change']['branch'], set()))
+            if data['type'] == 'ref-updated':
+                channel_set = self.channel_config.events.get('ref-updated')
+            else:
+                channel_set = (self.channel_config.projects.get(
+                    data['change']['project'], set()) &
+                    self.channel_config.events.get(
+                        data['type'], set()) &
+                    self.channel_config.branches.get(
+                        data['change']['branch'], set()))
         except KeyError:
             # The data we care about was not present, no channels want
             # this event.
             channel_set = set()
         self.log.info('Potential channels to receive event notification: %s' %
-                                                                  channel_set)
+                      channel_set)
         for channel in channel_set:
             if data['type'] == 'comment-added':
                 self.comment_added(channel, data)
@@ -213,6 +250,8 @@ class Gerrit(threading.Thread):
                 self.patchset_created(channel, data)
             elif data['type'] == 'change-merged':
                 self.change_merged(channel, data)
+            elif data['type'] == 'ref-updated':
+                self.ref_updated(channel, data)
 
     def run(self):
         while True:
@@ -222,7 +261,7 @@ class Gerrit(threading.Thread):
                 event = self.gerrit.getEvent()
                 self.log.info('Received event: %s' % event)
                 self._read(event)
-            except:
+            except Exception:
                 self.log.exception('Exception encountered in event loop')
                 if not self.gerrit.watcher_thread.is_alive():
                     # Start new gerrit connection. Don't need to restart IRC
@@ -257,7 +296,7 @@ class ChannelConfig(object):
 
 
 def _main():
-    config = ConfigParser.ConfigParser()
+    config = ConfigParser.ConfigParser({'server_password': None})
     config.read(sys.argv[1])
     setup_logging(config)
 
@@ -275,7 +314,8 @@ def _main():
                     config.get('ircbot', 'nick'),
                     config.get('ircbot', 'pass'),
                     config.get('ircbot', 'server'),
-                    config.getint('ircbot', 'port'))
+                    config.getint('ircbot', 'port'),
+                    config.get('ircbot', 'server_password'))
     g = Gerrit(bot,
                channel_config,
                config.get('gerrit', 'host'),
@@ -292,7 +332,7 @@ def main():
         sys.exit(1)
 
     pid = pid_file_module.TimeoutPIDLockFile(
-            "/var/run/gerritbot/gerritbot.pid", 10)
+        "/var/run/gerritbot/gerritbot.pid", 10)
     with daemon.DaemonContext(pidfile=pid):
         _main()
 
